//*****************************************************************************
// Description     : Main sample program of list mode
// Target Device   : APV8016A
// Target Platform : Linux, C
// Date            : Jan. 2021
// Version         : 1.0
//*****************************************************************************
#include <stdio.h>
#include <arpa/inet.h>

#include "config.h"
#include "file.h"
#include "sitcp.h"
#include "device.h"
#include "local_define.h"

//
// Local functions declaration
//
static void run_list_measurement(int, int, unsigned qword);
static int receive_list(int);


//#########################################################################
//  Name      : main
//  Function  : entry point
//  Arguments :
//      in argc : number of parameters
//      in argv : pointer to array of parameters
//  Return :
//      0 : success
//      1 : failure
//#########################################################################
int main(int argc, char **argv)
{
	int command_socket, data_socket;
	int ret;
	int run_ready = 0;
	unsigned qword meas_count;

	// 0. delete data file if exists
	unlink(LIST_FILENAME);

	// 1. connect to the device via SiTCP (both command and data)
	ret = sitcp_cmd_init(DEVICE_PORT_CTRL_OWNPC, &command_socket);
	if (ret < 0) {
		printf("connect command error(%d)\n", ret);
		return(1);
	}
	ret = sitcp_data_init(DEVICE_IPADDR, &data_socket);
	if (ret < 0) {
		printf("connect data error(%d)\n", ret);
		return(2);
	}

	// 2. set parameters to the device
	meas_count = device_msec2count(MEASUREMENT_TIME_MSEC);	// convert msec to count
	ret = config_device(command_socket, meas_count, DEVICE_PARAM_MOD_LIST);
	if (ret < 0) {
		printf("device config error(%d)\n", ret);
	}
	else {
		run_ready = 1;
	}

	if (run_ready == 1) { // ready to run
		// 3. start measurement
		printf("start measurement!!!\n");
		run_list_measurement(command_socket, data_socket, meas_count);

		// 4. stop measurement
		sitcp_write_16bit(command_socket, DEVICE_COMMAND_AQS, DEVICE_PARAM_AQS_STOP);
		printf("stop measurement!!!\n");
	}

	// 5. disconnect from device
	close(command_socket);
	close(data_socket);

	return(0);
}


//#########################################################################
//  Name      : run_list_measurement
//  Function  : list measurement processing
//  Arguments :
//      in sock_cmd : file descriptor for command communication
//      in sock_dat : file descriptor for data transfer
//      in ms_count : measurement time (counts)
//  Return : none
//#########################################################################
static void run_list_measurement(int sock_cmd, int sock_dat, unsigned qword ms_count)
{
	int ret;
	int cnt = 0;
	int sum = 0;
	unsigned qword rtm_count;
	unsigned short status;
	unsigned int input_cps[DEVICE_MAX_CHNUM];
	unsigned int throughput_cps[DEVICE_MAX_CHNUM];

	// clear data
	sitcp_write_16bit(sock_cmd, DEVICE_COMMAND_CLR, 0);
	sitcp_write_16bit(sock_cmd, DEVICE_COMMAND_CLR, 1);
	sitcp_write_16bit(sock_cmd, DEVICE_COMMAND_CLR, 0);

	// wait for completion of clear processing
	msleep(200);

	// start measurement(issue stop command first, in case of in the middle of measurement)
	sitcp_write_16bit(sock_cmd, DEVICE_COMMAND_AQS, DEVICE_PARAM_AQS_STOP);
	sitcp_write_16bit(sock_cmd, DEVICE_COMMAND_AQS, DEVICE_PARAM_AQS_START);

	// display top 2 lines of status field to explain each column(first 4CHs only as example)
	printf("err  <tm(ms)>|        <input rate(cps) / throughput rate(cps)>       | total\n");
	printf("cnt    real  |     CH1     |     CH2     |     CH3     |     CH4     | list size\n");
	printf("-------------+-------------+-------------+-------------+-------------+----------\n");

	while(1) {
		if (cnt % 500 == 0) { // read status every 500 receiving list
			ret = device_read_status(sock_cmd, 4, &status, &rtm_count, input_cps, throughput_cps);
			if (ret > 0) {
				printf(" | %9d\n", sum);
				fflush(stdout);
			}
			// status is 'stopped' or measurement time elapsed
			if ((status == DEVICE_PARAM_AQS_STOP) || (rtm_count >= ms_count)) {
				break;
			}
		}

		// receive list data
		ret = receive_list(sock_dat);
		if (ret > 0) sum += ret;

		msleep(1);	// wait for 1ms
		cnt++;
	}
}


//#########################################################################
//  Name      : receive_list
//  Function  : receiving list data
//  Arguments :
//      in sock_dat : file descriptor for data transfer
//  Return :
//      positive valuse : received list size
//      -1 : failed to receive
//      -2 : failed to write file
//#########################################################################
static int receive_list(int sock_dat)
{
	int ret;
	int result = 0;
	char ldata[LIST_TXR_LEN];

	// receiving list data
	ret = recv(sock_dat, ldata, LIST_TXR_LEN, MSG_DONTWAIT);
	result = ret;

	if (ret > 0) {
		// write data to file(append)
		ret = file_write_binary_data(LIST_FILENAME, -1, ret, ldata);
		if (ret < 0) {
			// failed to write
			result = -2;
		}
	} else {
		// failed to receive data
		result = -1;
	}

	return(result);
}
