//*****************************************************************************
// Description     : Definition of functions for configure device
// Target Device   : APV8016A
// Target Platform : Linux, C
// Date            : Jan. 2021
// Version         : 1.0
//*****************************************************************************
#include "sitcp.h"
#include "device.h"

//#########################################################################
//  Name      : config_device
//  Function  : set parameters to device
//  Arguments :
//      in sock_cmd : file descriptor for command communication
//      in ms_count : measurement time (counts)
//      in mode     : measurement mode
//  Return :
//       0 : success
//      -n : failed at nth parameter
//  Note : in this sample, set same value to all CHs
//#########################################################################
int config_device(int sock_cmd, unsigned qword ms_count, int mode)
{
	int i, ret;

	// set for each CH
	for (i = 0; i < DEVICE_MAX_CHNUM; i++) {
		// analog coarse gain: x2
		ret = sitcp_write_16bit(sock_cmd, DEVICE_COMMAND_ACG + DEVICE_OFFSETADDR_CH * i,
											DEVICE_PARAM_ACG_x2);
		if (ret < 0) return(-1);

		// ADC gain: 8192
		ret = sitcp_write_16bit(sock_cmd, DEVICE_COMMAND_ADG + DEVICE_OFFSETADDR_CH * i,
											DEVICE_PARAM_ADG_8192);
		if (ret < 0) return(-2);

		// Fast diff: 20
		ret = sitcp_write_16bit(sock_cmd, DEVICE_COMMAND_FFD + DEVICE_OFFSETADDR_CH * i,
											DEVICE_PARAM_FFD_20);
		if (ret < 0) return(-3);

		// Fast integral: 20
		ret = sitcp_write_16bit(sock_cmd, DEVICE_COMMAND_FFI + DEVICE_OFFSETADDR_CH * i,
											DEVICE_PARAM_FFI_20);
		if (ret < 0) return(-4);

		// fast pole zero: 20
		ret = sitcp_write_16bit(sock_cmd, DEVICE_COMMAND_FPZ + DEVICE_OFFSETADDR_CH * i, 20);
		if (ret < 0) return(-5);

		// fast threshold: 20
		ret = sitcp_write_16bit(sock_cmd, DEVICE_COMMAND_FTH + DEVICE_OFFSETADDR_CH * i, 20);
		if (ret < 0) return(-6);

		// slow rise time: 6us(600 @1digit=10ns)
		ret = sitcp_write_16bit(sock_cmd, DEVICE_COMMAND_SFR + DEVICE_OFFSETADDR_CH * i, 600);
		if (ret < 0) return(-7);

		// slow peaking time: 6700ns(670 @1digit=10ns)
		ret = sitcp_write_16bit(sock_cmd, DEVICE_COMMAND_SFP + DEVICE_OFFSETADDR_CH * i, 670);
		if (ret < 0) return(-8);

		// slow pole zero: 684ns
		ret = sitcp_write_16bit(sock_cmd, DEVICE_COMMAND_SPZ + DEVICE_OFFSETADDR_CH * i, 684);
		if (ret < 0) return(-9);

		// slow threshold: 50
		ret = sitcp_write_16bit(sock_cmd, DEVICE_COMMAND_STH + DEVICE_OFFSETADDR_CH * i, 50);
		if (ret < 0) return(-10);

		// LLD: 50
		ret = sitcp_write_16bit(sock_cmd, DEVICE_COMMAND_LLD + DEVICE_OFFSETADDR_CH * i, 50);
		if (ret < 0) return(-11);

		// ULD: 8000
		ret = sitcp_write_16bit(sock_cmd, DEVICE_COMMAND_ULD + DEVICE_OFFSETADDR_CH * i, 8000);
		if (ret < 0) return(-12);

		// Pileup reject: off 
		ret = sitcp_write_16bit(sock_cmd, DEVICE_COMMAND_PUR + DEVICE_OFFSETADDR_CH * i, 0);
		if (ret < 0) return(-13);

		// polarity: inverse
		ret = sitcp_write_16bit(sock_cmd, DEVICE_COMMAND_POL + DEVICE_OFFSETADDR_CH * i,
											DEVICE_PARAM_POL_INV);
		if (ret < 0) return(-14);

		// digital coarse gain: x4
		ret = sitcp_write_16bit(sock_cmd, DEVICE_COMMAND_DCG + DEVICE_OFFSETADDR_CH * i,
											DEVICE_PARAM_DCG_x4);
		if (ret < 0) return(-15);

		// digital fine gain: x0.7796 (0.7796 x 8193 - 2 = 6385)
		ret = sitcp_write_16bit(sock_cmd, DEVICE_COMMAND_DFG + DEVICE_OFFSETADDR_CH * i, 6385);
		if (ret < 0) return(-16);

		// timing select: CFD
		ret = sitcp_write_16bit(sock_cmd, DEVICE_COMMAND_TMS + DEVICE_OFFSETADDR_CH * i,
											DEVICE_PARAM_TMS_CFD);
		if (ret < 0) return(-17);

		// CFD function: 0.25
		ret = sitcp_write_16bit(sock_cmd, DEVICE_COMMAND_CFF + DEVICE_OFFSETADDR_CH * i,
											DEVICE_PARAM_CFF_0_25);
		if (ret < 0) return(-18);

		// CFD delay: 40ns
		ret = sitcp_write_16bit(sock_cmd, DEVICE_COMMAND_CFD + DEVICE_OFFSETADDR_CH * i,
											DEVICE_PARAM_CFD_40);
		if (ret < 0) return(-19);

		// inhibit width: 70ns(7 @1digit=10ns)
		ret = sitcp_write_16bit(sock_cmd, DEVICE_COMMAND_IHW + DEVICE_OFFSETADDR_CH * i, 7);
		if (ret < 0) return(-20);

		// analog pole zero: 0(auto)
		ret = sitcp_write_16bit(sock_cmd, DEVICE_COMMAND_PZD + DEVICE_OFFSETADDR_CH * i, 0);
		if (ret < 0) return(-21);

		// list wave compress:  10ns
		ret = sitcp_write_16bit(sock_cmd, DEVICE_DOMMAND_LWC + DEVICE_OFFSETADDR_CH * i, DEVICE_PARAM_LWC_10);
		if (ret < 0) return(-22);

		// list wave delay: 50ns (50 / 10 = 5)
		ret = sitcp_write_16bit(sock_cmd, DEVICE_DOMMAND_LWD + DEVICE_OFFSETADDR_CH * i, 5);
		if (ret < 0) return(-23);

		// list wave data: 500
		ret = sitcp_write_16bit(sock_cmd, DEVICE_DOMMAND_LWN + DEVICE_OFFSETADDR_CH * i, 500);
		if (ret < 0) return(-24);

		// list format: List
		ret = sitcp_write_16bit(sock_cmd, DEVICE_DOMMAND_LFS + DEVICE_OFFSETADDR_CH * i, DEVICE_PARAM_LFS_LIST);
		if (ret < 0) return(-25);

		// baseline restorer select: Middle
		ret = sitcp_write_16bit(sock_cmd, DEVICE_DOMMAND_BRS + DEVICE_OFFSETADDR_CH * i, DEVICE_PARAM_BRS_MID);

		// baseline count manual: 50
		ret = sitcp_write_16bit(sock_cmd, DEVICE_DOMMAND_BRM + DEVICE_OFFSETADDR_CH * i, 50);
		if (ret < 0) return(-26);
	}
	// write wait number
	ret = sitcp_write_16bit(sock_cmd, DEVICE_COMMAND_WWN, 4);
	if (ret < 0) return(-27);

	// DAC monitor: CH1 preamp
	ret = sitcp_write_16bit(sock_cmd, DEVICE_COMMAND_DAC, DEVICE_PARAM_DAC_CH1_PAMP);
	if (ret < 0) return(-28);

	// measurement mode
	ret = sitcp_write_16bit(sock_cmd, DEVICE_COMMAND_MOD, mode);
	if (ret < 0) return(-29);

	// measurement time
	ret = sitcp_write_48bit(sock_cmd, DEVICE_COMMAND_MTM, ms_count);
	if (ret < 0) return(-30);

	for (i = 0; i < DEVICE_MAX_CHNUM; i++) {
		// reset filter: 0,1,0
		ret = sitcp_write_16bit(sock_cmd, DEVICE_COMMAND_RFL + DEVICE_OFFSETADDR_CH * i, 0);
		ret += sitcp_write_16bit(sock_cmd, DEVICE_COMMAND_RFL + DEVICE_OFFSETADDR_CH * i, 1);
		ret += sitcp_write_16bit(sock_cmd, DEVICE_COMMAND_RFL + DEVICE_OFFSETADDR_CH * i, 0);
		if (ret < 0) return(-31);
	}
	return(0);
}
