//*****************************************************************************
// Description     : Definition of IO functions
// Target Device   : APV8016A
// Target Platform : Linux, C
// Date            : Nov 2020
// Version         : 1.0
//*****************************************************************************
#include <stdio.h>

#include "file.h"


//#########################################################################
//  Name      : file_write_text_header
//  Function  : write header to the text file
//  Arguments :
//      in pName    : file name
//      in real_time: real time(msec)
//      in ch_num   : number of CHs
//      in pICR     : pointer to the array of input count rate(cps)
//      in pTCR     : pointer to the array of throughput count rate(cps)
//  Return :
//       0 : success
//      -1 : failed to open file
//#########################################################################
int file_write_text_header(char *pName, unsigned int real_time,
					  int ch_num, unsigned int *pICR, unsigned int *pTCR)
{
	FILE *fp;
	int i;

	// open file(create)
	fp = fopen(pName, "wt");
	if (fp == NULL) {
		return(-1);
	}

	// write header information
	fprintf(fp, "[header]\n");
	fprintf(fp, "Real time(s),%d\n", real_time/1000);
	fprintf(fp, "Input rate(cps)");
	for(i = 0; i < ch_num; i++) {
		fprintf(fp, ",%d", *(pICR + i));
	}
	fprintf(fp, "\nThroughput rate(cps)");
	for(i = 0; i < ch_num; i++) {
		fprintf(fp, ",%d", *(pTCR + i));
	}
	fprintf(fp, "\n");

	// close file
	fclose(fp);

	return(0);
}


//#########################################################################
//  Name      : file_write_text_data
//  Function  : write data to the text file
//  Arguments :
//      in pName : file name
//      in chnum : number of CHs
//      in pBuff : array of data to be written
//  Return :
//       0 : success
//      -1 : failed to open file
//      -2 : failed to seek
//#########################################################################
int file_write_text_data(char *pName, int chnum, unsigned int pBuff[][NUM_DATA])
{
	FILE *fp;
	int ret, i, j;

	// open file(append)
	fp = fopen(pName, "rt+");
	if (fp == NULL) {
		return(-1);
	}

	// move to end of the file
	ret = fseek(fp, (int)0, SEEK_END);
	if (ret != 0) {
		fclose(fp);
		return(-2);
	}

	// write data
	fprintf(fp, "[data]\n");
	for (i = 0; i < NUM_DATA; i++) {
		fprintf(fp, "%d", pBuff[0][i]);         // CH1
		for (j = 1; j < chnum; j++) {
			fprintf(fp, ",%d", pBuff[j][i]);    // CH2-
		}
		fprintf(fp, "\n");
	}

	// close file
	fclose(fp);

	return(i);
}


//#########################################################################
//  Name      : file_write_binary_data
//  Function  : write data to the binary file
//  Arguments :
//      in pName : file name
//      in nseek : 0- write start position in byte
//                 -1 tail of the file
//      in nbyte : size of data to be written
//      in pBuff : array of data to be written
//  Return :
//       0 : success
//      -1 : failed to open file
//      -2 : failed to seek
//#########################################################################
int file_write_binary_data(char *pName, int nseek, int nbyte, void *pBuff)
{
	FILE *fp;
	size_t abyte;
	int ret;
	int result = 0;
	
	// open file(append)
	fp = fopen(pName, "rb+");  
	
	if (fp == NULL) {
		// open file(new) if not exist
		fp = fopen(pName, "wb");
	}
	
	if (fp != NULL) {
		if (nseek >= 0) {
			// move to the specified position
			ret = fseek(fp, (long)(nseek), SEEK_SET);
		}
		else {
			// move to the tail of the file
			ret = fseek(fp, (int)(0), SEEK_END);
		}
		
		if (ret == 0) {
			// write data
			abyte = fwrite(pBuff, (size_t)(1), (size_t)(nbyte), fp);
			result = (int)(abyte);
		}
		else {
			// faield to move
			result = -2;
		}

		fclose(fp);
	}
	else {
		// failed to open
		result = -1;
	}

	return(result);
}

