//*****************************************************************************
// Description     : Definition of functions for SiTCP communication
// Target Device   : APV8016A
// Target Platform : Linux, C
// Date            : Jan. 2021
// Version         : 1.0
//*****************************************************************************
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>

#include "sitcp.h"
#include "sys_compatible.h"
#include "command.h"
#include "local_define.h"


#define IPADDR_OWNPC			"0.0.0.0"	// IP address of PC; Normally leave 0.0.0.0
#define DEVICE_PORT_CTRL_DEVICE	(4660)	// Destination port number for command
#define DEVICE_PORT_DATARECV	(24)	// Port number for data reception

//
// Size of the buffer of socket
//
#define DEVICE_SOCKBUF_CMD	(16 * 1024)		// for receiving command response(16kB)
#define DEVICE_SOCKBUF_DATA	(1024 * 1024)	// for receiving data(1MB)

//
// Timeout value
//
#define DEVICE_TIMEOUT_CMD	(200)	// for waiting command response

//
// Local functions declaration
//
static int sitcp_read(int, unsigned int, unsigned short *);
static int sitcp_write(int, unsigned int, unsigned short);
static int sitcp_recv_cmd(int, char *, int, int);


//#########################################################################
//  Name      : sitcp_cmd_init
//  Function  : set up socket for command communication(UDP)
//  Arguments :
//      in  portn : port number for receiving response
//      out pSock : pointer to the file descriptor for the socket
//  Return :
//       0 : success
//      -1 : failed to create socket
//      -2 : bind error
//#########################################################################
int sitcp_cmd_init(unsigned short portn, int *pSock)
{
	struct sockaddr_in saddr;
	int sock;
	int ret;
	int result;
	int optval;
	socklen_t socklen;

	result = 0;
	*pSock = 0;

	// create UDP socket
	sock = socket(AF_INET, SOCK_DGRAM, 0);

	if (sock > 0) { // socket created
		// change maximum socket receive buffer size
		optval = DEVICE_SOCKBUF_CMD;
		socklen = sizeof(optval);
		ret = setsockopt(sock, SOL_SOCKET, SO_RCVBUF, &optval, socklen);

		// assign own address to the socket
		memset(&saddr, 0, sizeof(saddr));
		saddr.sin_family = AF_INET;
		saddr.sin_port = htons(portn);	// port number for receive
		saddr.sin_addr.s_addr = inet_addr(IPADDR_OWNPC);	// IP address of PC
		ret = bind(sock, (struct sockaddr*)(&saddr), sizeof(saddr));

		if (ret == 0) { // bind succeeded
			*pSock = sock;
			result = 0;
		}
		else { // bind error
			close(sock);
			result = -2;
		}
	}
	else { // failed to create socket
		result = -1;
	}

	return(result);
}


//#########################################################################
//  Name      : sitcp_data_init
//  Function  : set up socket for fast data transfer(TCP)
//  Arguments :
//      in  pIpAddr : IP address of target device
//      out pSock   : pointer to the file descriptor for the socket
//  Return :
//       0 : success
//      -1 : failed to create socket
//      -2 : connect error
//#########################################################################
int sitcp_data_init(char *pIpAddr, int *pSock)
{
	struct sockaddr_in saddr;
	int sock;
	int ret;
	int result;
	int optval;
	socklen_t socklen;

	result = 0;
	*pSock = 0;

	// create TCP socket
	sock = socket(AF_INET, SOCK_STREAM, 0);

	if (sock > 0) { // socket created
		// change maximum socket receive buffer size
		optval = DEVICE_SOCKBUF_DATA;
		socklen = sizeof(optval);
		ret = setsockopt(sock, SOL_SOCKET, SO_RCVBUF, &optval, socklen);

		// connect to target device
		memset(&saddr, 0, sizeof(saddr));
		saddr.sin_family = AF_INET;
		saddr.sin_port = htons(DEVICE_PORT_DATARECV);	// port number for receiving data
		saddr.sin_addr.s_addr = inet_addr(pIpAddr);	// IP address of target device
		ret = connect(sock, (struct sockaddr*)(&saddr), sizeof(saddr));

		if (ret == 0) { // connect succeeded
			*pSock = sock;
			result = 0;
		}
		else { // connect error
			close(sock);
			result = -2;
		}
	}
	else { // failed to create socket
		result = -1;
	}

	return(result);
}


//#########################################################################
//  Name      : sitcp_read
//  Function  : command communication(read request)
//  Arguments :
//      in  sock_cmd : file descriptor for command communication
//      in  cmd_addr : command address
//      out pData    : pointer to the response data
//  Return :
//       0 : success
//      -1 : no response within a period of timeout value
//      -2 : invalid size of received packet
//      -3 : received packet is not a response to the request
//  Note : This function doesn't convert received data. It's up to caller.
//#########################################################################
int sitcp_read(int sock_cmd, unsigned int cmd_addr, unsigned short *pData)
{
	struct sockaddr_in saddr;
	SITCP_FORMAT_ADDR_ONLY reqdt;
	SITCP_FORMAT_WITH_PARAM rcvdt;
	SITCP_HEADER cmpdt;
	int result;
	int sbyte, rbyte;
	int ssize, rsize;

	result = 0;

	// clear area for reception
	memset(&rcvdt, 0, sizeof(rcvdt));

	// set size of packet for request and response
	ssize = SITCP_FORMAT_LEN_ADDR_ONLY;
	rsize = SITCP_FORMAT_LEN_WITH_PARAM;

	// construct packet for read request
	memset(&reqdt, 0, sizeof(reqdt));
	reqdt.header.ver_type = SITCP_VER_TYPE;
	reqdt.header.cmd_flag = SITCP_READ_REQ_CMDFLAG;
	reqdt.header.id = SITCP_READ_ID;
	reqdt.header.length = SITCP_DATA_LEN;
	reqdt.address = htonl(cmd_addr);

	// set target address information
	memset(&saddr, 0, sizeof(saddr));
	saddr.sin_family = AF_INET;
	saddr.sin_port = htons(DEVICE_PORT_CTRL_DEVICE);	// target port
	saddr.sin_addr.s_addr = inet_addr(DEVICE_IPADDR);	// target IP address

	// send the packet to the target
	sbyte = sendto(sock_cmd, (char*)(&reqdt), ssize, 0,
				  (struct sockaddr*)(&saddr), sizeof(saddr));

	if (sbyte > 0) {
		// receive response
		rbyte = sitcp_recv_cmd(sock_cmd, (char*)(&rcvdt), rsize, DEVICE_TIMEOUT_CMD);

		if (rbyte >= rsize) {
			// construct header part of response packet from request one
			// by change flag, which is only difference, from 'request' to 'acknowledge'
			cmpdt = reqdt.header;
			cmpdt.cmd_flag = SITCP_READ_ACK_CMDFLAG;

			// check if header and address is expected one
			if ( (memcmp(&cmpdt, &rcvdt.header, sizeof(cmpdt)) != 0) || (reqdt.address != rcvdt.address) ) {
				result = -3;	// not a response to the request
			}
		}
		else { // short of data size
			result = -2;
		}
	}
	else { // receive error
		result = -1;
	}

	// pass response data to caller
	*pData = htons(rcvdt.param);

	return(result);
}


//#########################################################################
//  Name      : sitcp_read_16bit
//  Function  : command communication(read request for 16bit value)
//  Arguments :
//      in  sock_cmd : file descriptor for command communication
//      in  cmd_addr : command address
//      out errcd    : error info; see return value of sitcp_read
//  Return : read value or 0(in case of error)
//#########################################################################
unsigned short sitcp_read_16bit(int sock_cmd, unsigned int cmd_addr, int *errcd)
{
	int ret;
	unsigned short result = 0;

	ret = sitcp_read(sock_cmd, cmd_addr, &result);

	if (errcd != NULL) {
		*errcd = ret;
	}

	return(result);
}


//#########################################################################
//  Name      : sitcp_read_32bit
//  Function  : command communication(read request for 32bit value)
//  Arguments :
//      in  sock_cmd : file descriptor for command communication
//      in  cmd_addr : command address
//      out errcd    : error info; see return value of sitcp_read
//  Return : read value or 0(in case of error)
//#########################################################################
unsigned int sitcp_read_32bit(int sock_cmd, unsigned int cmd_addr, int *errcd)
{
	int i, ret;
	unsigned short rdat[2];
	unsigned int result = 0;

	memset(&rdat, 0, sizeof(rdat));

	// read data in each 16bits (start with MSB)
	for (i = 0; i < 2; i++) {
		ret = sitcp_read(sock_cmd, cmd_addr + i*2, &rdat[i]);
		if (ret == 0) {
			result += (unsigned int)( (unsigned int)rdat[i] << (1-i)*16 );
		} else {
			break;
		}
	}

	if (errcd != NULL) {
		*errcd = ret;
	}

	return(result);
}


//#########################################################################
//  Name      : sitcp_read_48bit
//  Function  : command communication(read request for 48bit value)
//  Arguments :
//      in  sock_cmd : file descriptor for command communication
//      in  cmd_addr : command address
//      out errcd    : error info; see return value of sitcp_read
//  Return : read value or 0(in case of error)
//#########################################################################
unsigned qword sitcp_read_48bit(int sock_cmd, unsigned int cmd_addr, int *errcd)
{
	int i, ret;
	unsigned short rdat[3];
	unsigned qword result = 0;

	memset(&rdat, 0, sizeof(rdat));

	// read data in each 16bits (start with MSB)
	for (i = 0; i < 3; i++) {
		ret = sitcp_read(sock_cmd, cmd_addr + i*2, &rdat[i]);
		if (ret == 0) {
			result += (unsigned qword)( (unsigned qword)rdat[i] << (2-i)*16 );
		} else {
			break;
		}
	}

	if (errcd != NULL) {
		*errcd = ret;
	}

	return(result);
}


//#########################################################################
//  Name      : sitcp_write
//  Function  : command communication(write request)
//  Arguments :
//      in  sock_cmd : file descriptor for command communication
//      in  cmd_addr : command address
//      in  cmd_pram : parameter to be written
//  Return :
//       0 : success
//      -1 : no response within a period of timeout value
//      -2 : invalid size of received packet
//      -3 : received packet is not a response to the request
//#########################################################################
int sitcp_write(int sock_cmd, unsigned int cmd_addr, unsigned short cmd_param)
{
	struct sockaddr_in saddr;
	SITCP_FORMAT_WITH_PARAM reqdt, rcvdt;
	SITCP_HEADER cmpdt;
	int result;
	int sbyte, rbyte;
	int ssize, rsize;

	result = 0;

	// clear area for reception
	memset(&rcvdt, 0, sizeof(rcvdt));

	// set size of packet for request and response
	ssize = SITCP_FORMAT_LEN_WITH_PARAM;
	rsize = ssize;

	// construct packet for write request
	reqdt.header.ver_type = SITCP_VER_TYPE;
	reqdt.header.cmd_flag = SITCP_WRITE_REQ_CMDFLAG;
	reqdt.header.id = SITCP_WRITE_ID;
	reqdt.header.length = SITCP_DATA_LEN;
	reqdt.address = htonl(cmd_addr);
	reqdt.param = htons(cmd_param);

	// set target address information
	memset(&saddr, 0, sizeof(saddr));
	saddr.sin_family = AF_INET;
	saddr.sin_port = htons(DEVICE_PORT_CTRL_DEVICE);	// target port
	saddr.sin_addr.s_addr = inet_addr(DEVICE_IPADDR);	// target IP address

	// send the packet to the target
	sbyte = sendto(sock_cmd, (char*)(&reqdt), ssize, 0,
				  (struct sockaddr*)(&saddr), sizeof(saddr));

	if (sbyte > 0) {
		// receive response
		rbyte = sitcp_recv_cmd(sock_cmd, (char*)(&rcvdt), rsize, DEVICE_TIMEOUT_CMD);

		if (rbyte >= rsize) {
			// construct header part of response packet from request one
			// by change flag, which is only difference, from 'request' to 'acknowledge'
			cmpdt = reqdt.header;
			cmpdt.cmd_flag = SITCP_WRITE_ACK_CMDFLAG;

			// check if header and address is expected one
			if ( (memcmp(&cmpdt, &rcvdt.header, sizeof(cmpdt)) != 0) || (reqdt.address != rcvdt.address) ){
				result = -3;	// not a response to the request
			}
		}
		else { // short of data size
			result = -2;
		}
	}
	else { // receive error
		result = -1;
	}

	return(result);
}


//#########################################################################
//  Name      : sitcp_write_16bit
//  Function  : command communication(write request for 16bit value)
//  Arguments :
//      in sock_cmd : file descriptor for command communication
//      in cmd_addr : command address
//      in wdata    : data to be written
//  Return : same as sitcp_write
//#########################################################################
int sitcp_write_16bit(int sock_cmd, unsigned int cmd_addr, unsigned short wdata)
{
	int result;

	result = sitcp_write(sock_cmd, cmd_addr, wdata);
	return(result);
}


//#########################################################################
//  Name      : sitcp_write_48bit
//  Function  : command communication(write request for 48bit value)
//  Arguments :
//      in sock_cmd : file descriptor for command communication
//      in cmd_addr : command address
//      in wdata    : data to be written
//  Return : same as sitcp_write
//#########################################################################
int sitcp_write_48bit(int sock_cmd, unsigned int cmd_addr, unsigned qword wdata)
{
	int i, result;
	unsigned short param;

	// write data in each 16bits (start with MSB)
	for(i = 0; i < 3; i++) {
		param = (unsigned short)( 0xFFFF & (wdata >> ((2-i)*16)) );
		result = sitcp_write(sock_cmd, cmd_addr + i*2, param);
		if (result < 0) break;
	}

	return(result);
}


//#########################################################################
//  Name      : sitcp_recv_cmd
//  Function  : command communication(receive response)
//  Arguments :
//      in sock    : file descriptor for command communication
//      in rbuffer : buffer for received data
//      in size    : size of data to be received(byte)
//      in timeout : receive timeout value(msec)
//  Return :
//      positive value: received data size
//      -11 : no response within a period of timeout value
//      -12 : no data
//#########################################################################
int sitcp_recv_cmd(int sock, char *rbuffer, int size, int timeout)
{
	struct sockaddr_in saddr;
	struct timeval tval;
	socklen_t sock_len;
	fd_set fdsock;
	int ret;
	int result;
	int recv_size;

	result = 0;

	// set timeout value
	memset(&tval, 0, sizeof(tval));
	tval.tv_sec = (int)(timeout / 1000);
	tval.tv_usec = (int)((timeout % 1000) * 1000);

	// wait until received data is ready
	FD_ZERO(&fdsock);
	FD_SET(sock, &fdsock);
	ret = select((sock + 1), &fdsock, NULL, NULL, &tval);

	if (ret != 0) {
		if (FD_ISSET(sock, &fdsock) != 0) {
			// receive message
			sock_len = sizeof(saddr);
			memset(&saddr, 0, sizeof(saddr));
			recv_size = recvfrom(sock, rbuffer, size, 0, (struct sockaddr*)(&saddr), &sock_len);
			result = (int)(recv_size);	// set received data size
		}
		else { // no data
			result = -12;
		}
	}
	else { // timeout
		result = -11;
	}

	return(result);
}


//#########################################################################
//  Name      : sitcp_ntoh64
//  Function  : convert 64bit value from network byte order to host byte order 
//  Arguments :
//      in value : 64bit value in network byte order
//  Return : 64bit value in host byte order
//#########################################################################
unsigned qword sitcp_ntoh64(unsigned qword value)
{
	return( sitcp_hton64(value) );
}


//#########################################################################
//  Name      : sitcp_hton64
//  Function  : convert 64bit value from host byte order to network byte order
//  Arguments :
//      in value : 64bit value in host byte order
//  Return : 64bit value in network byte order
//#########################################################################
unsigned qword sitcp_hton64(unsigned qword value)
{
	unsigned char *p, *q;
	unsigned qword result;
	const static char adefs[2] = { 0x12, 0x34 };	// for identifying endian

	result = 0;

	if (*(short*)(adefs) != 0x1234) { // little endian
		// invert byte order
		p = (unsigned char*)(&result);
		q = (unsigned char*)(&value);

		p[0] = q[7];
		p[1] = q[6];
		p[2] = q[5];
		p[3] = q[4];
		p[4] = q[3];
		p[5] = q[2];
		p[6] = q[1];
		p[7] = q[0];
	}
	else { // big endian; no need to convert
		result = value;
	}

	return(result);
}
